
# q-expansion of newform 2156.1.k.c, downloaded from the LMFDB on 21 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as list of pairs
# Each pair (c, e) corresponds to c*zeta^e

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import CyclotomicField
    K = CyclotomicField(poly_data, "z")
    convert_elt_to_field = lambda elt: sum(c * K.gens()[0]**e for c,e in elt)
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2156
weight = 1
poly_data = 12
hecke_ring_character_values = [[1079, [[1, 0]]], [1277, [[1, 4]]], [981, [[-1, 0]]]]
aps_data = [[], [[1, 1], [1, 3]], [[1, 3], [1, 5]], [], [[-1, 2]], [], [], [], [[1, 4]], [], [[1, 1], [1, 3]], [[-1, 4]], [], [], [], [[1, 2], [1, 2]], [[-1, 1], [-1, 3]], [], [[-1, 2]], [[1, 0]], [], [], [], [[1, 3], [1, 5]], [[1, 1], [-1, 5]], [], [], [], [], [[-1, 0]], [], [], [[1, 2]], [], [], [], [[1, 1], [1, 3]], [[-1, 4], [-1, 4]], [], [], [[1, 2]], [[-1, 1], [1, 5]], [[-1, 4]], [], [], [], [], [[1, 1], [-1, 5]], [], [[1, 3], [1, 5]], [], [], [], [[-1, 1], [1, 5]], [], [], [], [], [], [], [], [], [], [], [[-1, 3], [-1, 5]], [[-1, 4]], [[-1, 4]], [], [], [], [[1, 1], [1, 3]], [], [[-1, 1], [-1, 3]], [], [[1, 0]], [[-1, 3], [-1, 5]], [[-1, 2]], [], [[-1, 4], [-1, 4]], [], [], [[-1, 0], [-1, 0]], [], [[1, 1], [-1, 5]], [], [[-1, 4]], [[1, 0]], [], [], [[1, 0]], [[-1, 3], [-1, 5]], [], [[-1, 2]], [], [[-1, 4], [-1, 4]], [], [[-1, 3], [-1, 5]], [[1, 1], [1, 3]], [], [], [], [], [], [], [], [[-1, 1], [-1, 3]], [], [], [[1, 2], [1, 2]], [], [], [], [[1, 0], [1, 0]], [[-1, 1], [-1, 3]], [[-1, 0]], [[1, 2]], [[-1, 1], [1, 5]], [[-1, 1], [-1, 3]], [[1, 4]], [], [[1, 1], [1, 3]], [], [], [[-1, 2], [-1, 2]], [[1, 3], [1, 5]], [], [[1, 4]], [[-1, 3], [-1, 5]], [[-1, 1], [1, 5]], [], [], [], [[1, 4]], [[-1, 0], [-1, 0]], [], [], [], [], [[1, 1], [-1, 5]], [], [], [], [[1, 2]], [], [[-1, 1], [-1, 3]], [[-1, 1], [1, 5]], [], [], [[-1, 3], [-1, 5]], [[1, 4], [1, 4]], [], [[-1, 1], [1, 5]], [[1, 0], [1, 0]], [], [[1, 2], [1, 2]], [[-1, 0], [-1, 0]], [], [], [], [], [[-1, 4]], [], [], [[1, 3], [1, 5]], [[-1, 2]], [[-1, 1], [-1, 3]], [[1, 2], [1, 2]], [], [], [[-1, 3], [-1, 5]], [], [], [], [], [[1, 1], [1, 3]], [[-1, 1], [1, 5]], [], [[1, 2]], [], [], [[1, 4], [1, 4]], [], [[1, 0]], [], [[1, 2]], [], [], [], [], [], [[1, 3], [1, 5]], [], [[1, 4]], [], [], [[-1, 1], [-1, 3]], [], [[-1, 4]], [], [], [], [], [], [], [[1, 1], [-1, 5]], [[-1, 1], [-1, 3]], [[-1, 3], [-1, 5]], [], [], [], [], [[-1, 1], [1, 5]], [[-1, 0]], [], [], [[-1, 3], [-1, 5]], [], [], [[-1, 4], [-1, 4]], [[1, 0]], [], [], [[1, 4]], [[-1, 4]], [], [], [], [[-1, 2], [-1, 2]], [], [], [[1, 2]], [], [], [], [[-1, 1], [1, 5]], [], [[-1, 3], [-1, 5]], [], [[1, 0]], [[1, 1], [-1, 5]], [[-1, 2]], [], [], [[1, 4]], [], [], [], [], [], [], [], [], [[-1, 2]], [], [], [], [[-1, 2], [-1, 2]], [], [[-1, 1], [1, 5]], [], [], [], [], [], [], [], [[1, 0]], [[-1, 1], [1, 5]], [], [], [[1, 3], [1, 5]], [[-1, 2]], [], [], [], [[1, 3], [1, 5]], [[-1, 4]], [], [], [], [], [[-1, 2]], [], [], [], [[1, 4]], [[1, 2], [1, 2]], [], [[1, 1], [1, 3]], [], [[-1, 2]], [], [], [], [], [], [[1, 3], [1, 5]], [[1, 1], [-1, 5]], [], [], [], [], []]
